% sensitivity of dissipation rate to dSdz, dTdz

% close all
% clear

%% define constant values
Sv = -80;                       % choose the noise threshold of your acoustic system here
sigma_v = 10.^(Sv/10);          % convert from dB to log space

% acoustic center frequency and wavenumber
f = 70000;                      % center frequency of the acoustic system
ss = 1420;                      % sound speed
k = 2*pi*f/ss;                  % = 310 1/m with f=70 kHz center frequency 

% oceanographic constants
lat = 18;                       % latitude
p = 50;                         % pressure (what depth are you looking at)

N2 = [];                        % buoyancy frequency
SA_0 = 6;                       % background salinity
CT_0 = 3;                       % background temperature

rho_0 = gsw_rho(SA_0,CT_0,p);   % background density
nu = 1.6*1e-6;                  % water viscosity

% calculate thermodynamic constants 
[A,B,alpha, beta, a, b] = TD_coeff_AB(CT_0,SA_0,p);

% mixing and other constants
q = 3.7;                        % universal straining constant
g = 9.81;                       % gravity constant
gamma = 0.2;                    % mixing coefficient
kappa_T = 1.5*1e-7;             % diffusivity of temperature
kappa_S = 1.5*1e-9;             % diffusitivty of salt


%% create arrays with dT/dz and dS/dz
lin_or_log = 'lin'              % decide if you wanna plot on log scale or linear scale

% log scale
dTdz_negativ = false            % decide if you want to look at positive or negative temperature gradients

if lin_or_log =='log'
    grad_CT_range = logspace(-4,-1,100);
    grad_SA_range = logspace(-4,-1,100);

    %  if you wanna look at negative temperature gradients in log space
    if dTdz_negativ == true
        grad_CT_range = -grad_CT_range;
    end
end

% linear scale
if lin_or_log == 'lin'
    grad_CT_range = linspace(-0.1,0.1,100);
    grad_SA_range = linspace(-0.03,0.1,100);
end 

% make grid out of gradient arrays
[grad_CT,grad_SA] = meshgrid(grad_CT_range,grad_SA_range);


%% calculations

% calculate N2 from T and S gradients
for i = 1:length(grad_CT_range)
    for j = 1:length(grad_SA_range)
         [N2(i,j)] = g/rho_0*(-alpha*grad_CT(i,j)+beta*grad_SA(i,j));
    end   
end

% calculate epsilon based on acoustic model without exponential decay
eps_echo = sigma_v.^2.*((8*N2).^2)/((q*gamma.*k).^2)./nu./(A.^2.*grad_CT.^2+B.^2.*grad_SA.^2+2.*A.*B.*grad_CT.*grad_SA).^2;

% calculate epsilon using full acoustic model, with method described in
% Ross et al. First define x as sqrt(eps/nu) and create array of values
eps_min = 1e-9;  
eps_max = 1e-4; 
x = logspace(log10(sqrt(eps_min/nu)),log10(sqrt(eps_max/nu)),10000);

sigma_inv = NaN(1,length(x));
sigma_inv_M = [];
double(sigma_inv);
eps_echo_dec = NaN(length(grad_CT_range),length(grad_SA_range));

sigma_plot = [];

%% plot sigma_v as function of epsilon for each combination of dT/dz, dS/dz
% figure(11)
% set(gca, 'YScale', 'log');
% set(gca, 'XScale', 'log');
% yline(1e-8)
% hold all
% grid on
% xlabel([char(949) ' (W kg^-^1)']), ylabel('\sigma_V')
%%

M_ind_extr = NaN(length(grad_CT_range),length(grad_SA_range),10);

for i = 1:length(grad_CT_range)
    for j = 1:length(grad_SA_range)
        
        x = logspace(log10(sqrt(eps_min/nu)),log10(sqrt(eps_max/nu)),10000);
      
        temp_contr = A*grad_CT(i,j)*exp(-2*q*k^2*kappa_T./x);             
        sal_contr = B*grad_SA(i,j)*exp((-2*q*k^2*kappa_S)./x);     
        
        % calculate sigma which will then be used to estimate epsilon
        sigma_inv = x * (q*gamma*k)*nu/(8*N2(i,j)).*((temp_contr + sal_contr).^2);
       
    %   plot(x.^2*nu,sigma_inv, '.')
    %   pause
        
        % remove values larger than 1e-11 to find minima in sigma_inv
        % find first ind above Sv threshold
        [above_Sv above_Sv_inds] = find(sigma_inv >=1e-8);
        
        if isempty(above_Sv_inds)
            above_Sv_inds = 1;
        end
                
        sigma_inv_temp = sigma_inv(above_Sv_inds(1):end);
      
        local_extrema = islocalmin(sigma_inv_temp) | islocalmax(sigma_inv_temp);
        [val_min, ind_extr] = find(local_extrema == true);
       
       % if no minima was found or minima was found at the end of the array
       % use first value
       if isempty(ind_extr) || all(ind_extr) == length(sigma_inv)
           ind_extr = 1;
       end
       % write all inds in matrix
       M_ind_extr(i,j) = length(ind_extr);
       
       % cut sigma_inv and corresponding x vector from dip on
       sigma_inv = sigma_inv(ind_extr:end);
       x = x(ind_extr:end);
                       
       % search value of sigma_inv that is closest to the given sigma_v
       % threshold which we define at the beginning - the backscatter level
       % which can be detected with our system - for instance -80dB (10 dB
       % above the noise threshold of -90 dB)
       [val_sig, ind_sig_x] = min(abs(sigma_inv-sigma_v));
           
       % epsilon = x^2 * nu, using x closest to the one corresponding to
       % sigma_inv that's closest to sigma_v threshold
       eps_echo_dec(i,j) = x(ind_sig_x)^2*nu;
        
       
        %% look for multiple solutions
        % if there is a local minimum, reject the datapoint
        if any(local_extrema(2:end-1))== 1
            eps_echo_dec(i,j) = nan;
        end
        
        sigma_plot(i,j) = val_sig;

        %% if sigma_inv is negativ (unstable stratification) the theory
        % shouldn't be applied and we set the calculated value to nan
        if sigma_inv < 0
            eps_echo_dec(i,j) = nan;            
        end
        
        clear x sigma_inv val_sig ind_sig_x temp_contr sal_contr ind_min val_min sigma_inv_temp 
        clear above_Sv above_Sv_inds ind_extr  
    end
end
 

hold off
labelsize = 12

figure(42)

if lin_or_log == 'log'
    if dTdz_negativ == false
        pcolor((grad_CT_range), (grad_SA_range), (eps_echo_dec)); shading flat
    elseif dTdz_negativ == true
        pcolor((-grad_CT_range), (grad_SA_range), (eps_echo_dec)); shading flat
        xlabel('- d{\itT}/d{\itz} ( ^\circC m^-^1)')
    end
    set(gca, 'YScale', 'log');
    set(gca, 'XScale', 'log');

elseif lin_or_log == 'lin'
    pcolor((grad_CT_range), (grad_SA_range), (eps_echo_dec)); shading flat
    set(gca, 'YScale', 'lin');
    set(gca, 'XScale', 'lin'); 
end

xlabel('d{\itT}/d{\itz} ( ^\circC m^-^1)'), ylabel('d{\itS}/d{\itz} (g kg^-^1 m^-^1)')

h = colorbar
colormap(flipud('viridis'))
set(gca,'ColorScale','log')
xlabel(h, [char(949) ' (W kg^-^1)'],'FontSize',labelsize)
set(findall(gcf,'-property','FontSize'),'FontSize',labelsize);
grid on 
set(gca,'layer','top')

caxis([1e-9 1e-6])
% xlim([1e-4, 1e-1])
% ylim([1e-4, 1e-1])



